<?php

namespace Drupal\send_mails\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\send_mails\Utils\SendMailsHelper;

/**
 * Implements the SendMailsForm form controller.
 *
 * The form help to build mail composing interface.
 *
 * @see \Drupal\Core\Form\FormBase
 */
class SendMailsForm extends FormBase implements ContainerInjectionInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The Entity user storage.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $userStorage;

  /**
   * Constructs a new PaymentAddForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The inline form manager.
   * @param \Drupal\send_mails\Utils\SendMailsHelper $utils_helper
   *   The General Util.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $account, SendMailsHelper $utils_helper) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $account;
    $this->utilHelper = $utils_helper;
    $this->userRoles = $entity_type_manager->getStorage('user_role');
    $this->userStorage = $entity_type_manager->getStorage('user');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('send_mails.helper')
    );
  }

  /**
   * Build the Email form.
   *
   * A build form method constructs an array that defines how markup and
   * other form elements are included in an HTML form.
   *
   * @param array $form
   *   Default form array structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object containing current form state.
   *
   * @return array
   *   The render array defining the elements of the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $roles = [];
    foreach ($this->userRoles->loadMultiple() as $key => $role) {
      $roles[$key] = $role->label();
    }
    unset($roles['anonymous']);

    // Advanced send option fields.
    if ($this->currentUser->hasPermission('send_mails advanced')) {
      $form['advanced_send_option'] = [
        '#type' => 'details',
        '#title' => $this->t('Advanced Send option'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      ];
      $form['advanced_send_option']['role_based'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Role based recievers'),
      ];
      $form['advanced_send_option']['user_role'] = [
        '#type' => 'radios',
        '#options' => $roles,
        '#title' => $this->t('Select required role'),
        '#states' => [
          'visible' => [
            ':input[name="role_based"]' => ['checked' => TRUE],
          ],
        ],
      ];
    }

    $form['to_id'] = [
      '#type' => 'entity_autocomplete',
      '#attributes' => [
        'class' => ['container-inline'],
      ],
      '#target_type' => 'user',
      '#tags' => TRUE,
      '#selection_settings' => [
        'match_operator' => 'CONTAINS',
        'include_anonymous' => FALSE,
      ],
      '#size' => 60,
      '#maxlength' => 1000,
      '#title' => $this->t('To Address'),
      '#description' => $this->t("Enter the senders mail id separated by comma(,). Example: example@example.com,test@test.com,.."),
      '#states' => [
        'invisible' => [
          ':input[name="role_based"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="role_based"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['subject'] = [
      '#type' => 'textfield',
      '#size' => 60,
      '#maxlength' => 255,
      '#required' => TRUE,
      '#title' => $this->t('Mail Subject'),
      '#description' => $this->t("Enter the Subject for the mail"),
    ];

    $form['body'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Mail Body'),
      '#required' => TRUE,
      '#rows' => 20,
      '#cols' => 60,
      '#description' => $this->t("Enter the Body content for mail"),
      '#attributes' => ['class' => ['text-full']],
    ];

    $form['attachments'] = [
      '#title' => $this->t('Attachment'),
      '#type' => 'managed_file',
      '#multiple' => TRUE,
      '#upload_location' => 'public://send_mails/',
      '#description' => $this->t('Upload up to 5 MB files.'),
      '#upload_validators' => [
        'file_validate_extensions' => ['zip tar tgz taz z gz rar gif png jpg jpeg doc xls ppt pdf'],
        'file_validate_size' => [5000000],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;

  }

  /**
   * Getter method for Form ID.
   *
   * The form ID is used in implementations of hook_form_alter() to allow other
   * modules to alter the render array built by this form controller. It must be
   * unique site wide. It normally starts with the providing module's name.
   *
   * @return string
   *   The unique ID of the form defined by this class.
   */
  public function getFormId() {
    return 'send_mails_form';
  }

  /**
   * Implements form validation.
   *
   * The validateForm method is the default method called to validate input on
   * a form.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $mailids = [];
    // Role based To address.
    $values = $form_state->getValues();
    if ($values['role_based']) {
      $mailids = $this->utilHelper->getUsersByRole($values['user_role']);
      $mails = implode(",", $mailids);
    }
    else {
      foreach ($values['to_id'] as $user) {
        $mailids[] = $this->userStorage->load($user['target_id'])->getEmail();
      }
      $mails = implode(",", $mailids);
    }
    foreach ($mailids as $mail) {
      if (!filter_var(trim($mail), FILTER_VALIDATE_EMAIL)) {
        $form_state->setErrorByName('to_id', $this->t('Invalid "To Address" please enter valid e-mail ids'));
      }
    }
    $form_state->setValue('mails', $mails);
  }

  /**
   * Implements a form submit handler.
   *
   * The submitForm method is the default method called for any submit elements.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /*
     * This would normally be replaced by code that actually does something
     * with the title.
     */
    $values = $form_state->getValues();
    $this->utilHelper->sendMails($values['mails'], $values['subject'], $values['body'], $values['attachments']);
    $this->messenger()->addMessage($this->t('Successfully mail has been send.'));
  }

}
