<?php

namespace Drupal\send_mails\Utils;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;

/**
 * SendMailsHelper Helper function.
 */
class SendMailsHelper {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The database connection to be used.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * Language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The cron configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The File System service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs this SendMailsUtilis object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   Mail manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel
   *   The logger channel factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $database, MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_channel, FileSystemInterface $file_system) {
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
    $this->config = $config_factory;
    $this->logger = $logger_channel;
    $this->fileSystem = $file_system;
  }

  /**
   * Get User Email ID from the roles.
   */
  public function getUsersByRole($role) {
    $userlist = [];
    $userObj = $this->entityTypeManager->getStorage('user');
    $ids = $userObj->getQuery()
      ->condition('status', 1)
      ->condition('roles', $role)
      ->execute();
    $users = $userObj->loadMultiple($ids);
    foreach ($users as $user) {
      $userlist[] = $user->get('mail')->getString();
    }
    return $userlist;
  }

  /**
   * Trigger Email.
   */
  public function sendMails($to_ids, $subject, $body, $attachment) {
    $site_mail = $this->config->get('system.site')->get('mail');
    $from_mail = $site_mail;
    $params = [
      'subject' => $subject,
      'body' => $body['value'],
      'from' => $from_mail,
      'headers' => [
        'from' => $from_mail,
        'MIME-Version' => '1.0',
        'Content-Type' => 'text/html; charset=UTF-8; format=flowed; delsp=yes',
        'Content-Transfer-Encoding' => '8Bit',
        'Content-Disposition' => 'attachment;',
        'X-Mailer' => 'Drupal',
        'Return-Path' => $from_mail,
      ],
    ];

    if ($attachment) {
      $fileObj = $this->entityTypeManager->getStorage('file')->load($attachment[0]);
      $file = [
        'filepath' => $this->fileSystem->realpath($fileObj->getFileUri()),
      ];
      $params['attachments'][] = $file;
    }
    $default_lang = $this->languageManager->getDefaultLanguage()->getId();
    $to_ids = explode(",", $to_ids);
    foreach ($to_ids as $mail) {
      $this->mailManager->mail('send_mails', 'send_mails_send', $mail, $default_lang, $params, $from_mail);
    }
  }

}
